﻿local L = AceLibrary("AceLocale-2.2"):new("SacredBuff")
local BS = AceLibrary("Babble-Spell-2.2")
local M = AceLibrary("Metrognome-2.0")
local _G = getfenv(0)

local stateNames = {
	"singletarget",
	"grouptarget",
	"singleplayer",
	"groupplayer"
}

local hearthstone = GetItemInfo(6948)

local TEX_SERENITY = "Interface\\AddOns\\SacredBuff\\UI\\serenity0"
local TEX_SPHERE = "Interface\\AddOns\\SacredBuff\\UI\\Symbol32"

function SacredBuff:CreateTooltip()
	local f = CreateFrame("GameTooltip", "SacredBuffTooltip", UIParent, "GameTooltipTemplate")
	f:SetScript("OnLoad",function() this:SetOwner(WorldFrame, "ANCHOR_NONE") end)
	f:SetFrameStrata("LOW")
	f:SetToplevel(true)
	f:SetMovable(true)
	f:SetClampedToScreen(true)
	self.tooltip = f
end

function SacredBuff:CreateMainFrame()
	self.frame = CreateFrame("Frame", "SacredBuffFrame", UIParent, "SecureStateHeaderTemplate")
	local f = self.frame
	f:Hide()
	f:EnableMouse(true)
	f:SetMovable(true)
	f:SetHeight(128)
	f:SetWidth(128)
	f:SetFrameStrata("LOW")
	f:SetClampedToScreen(true)
	
	f.title = f:CreateFontString(nil, "ARTWORK")
	f.title:ClearAllPoints()
	f.title:SetPoint("BOTTOM", f, "TOP", 0, 0)
	f.title:SetFont("Fonts\\ARIALN.ttf", 11, "OUTLINE")
	f.title:SetText(L["SacredBuff"])
	f.title:SetTextColor(1.00, 1.00, 1.00, 1.00)
	
	f:SetBackdrop({ 
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
		edgeFile = "", tile = true, tileSize = 16, edgeSize = 16, 
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	})
	if self.db.profile.movable then
		f:SetBackdropColor(0.00, 0.00, 0.00, 0.60)
	else
		f:SetBackdropColor(0.00, 0.00, 0.00, 0.00)
		f.title:Hide()
	end
	
	f:SetAttribute("statebutton1", "1:singletarget;2:grouptarget")
	f:SetAttribute("statebutton2", "1:singleplayer;2:groupplayer")
	f:SetAttribute("state", self.db.char.groupMode)
	
	f:SetAttribute("unit-singletarget", ATTRIBUTE_NOOP)
	f:SetAttribute("unit-grouptarget", ATTRIBUTE_NOOP)
	f:SetAttribute("unit-singleplayer", "player")
	f:SetAttribute("unit-groupplayer", "player")
	
	f.StateChanged = function()
		self.db.char.groupMode = tonumber( f:GetAttribute("state") )
		self:TriggerEvent("SacredBuff_ModeChanged")
	end
	
	self:RestorePosition()
	self:ScaleFrame()
	
	f:SetScript("OnMouseDown", function()
		if arg1 == "LeftButton" and self.db.profile.movable == true then
			f:StartMoving()
			f.isMoving = true
		end
	end)
	f:SetScript("OnMouseUp", function()
		if this.isMoving then
			f:StopMovingOrSizing()
			self:SavePosition()
			f.isMoving = false
		end
	end)
	f:SetScript("OnHide", function()
		if this.isMoving then
			f:StopMovingOrSizing()
			f.isMoving = false
		end
	end)
	
	local t = CreateFrame("Button", "SacredBuffFrameMain", f, "SecureActionButtonTemplate")
	t:SetWidth(64)
	t:SetHeight(64)
	t:EnableMouse(true)
	t:SetFrameStrata("LOW")
	
	t:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	
	t:SetFont("Fonts\\ARIALN.ttf", 16, "OUTLINE")
	t:SetHighlightTextColor(1, 1, 0, 1)
	t:SetText(self.db.char.sacredCount)
	t:SetTextColor(1.00, 1.00, 1.00, 1.00)
	t:ClearAllPoints()
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	
	t:SetNormalTexture(TEX_SPHERE)
	
	t:SetAttribute("newstate1", "1,2")
	t:SetAttribute("*type2", "item")
	t:SetAttribute("item", hearthstone)
	f.main = t
	f:SetAttribute("addchild", f.main)
	
	t = f:CreateTexture(nil, "OVERLAY")
	t:SetWidth(64)
	t:SetHeight(64)
	t:SetTexture(TEX_SERENITY)
	t:SetPoint("CENTER", f.main, "CENTER", 0, 0)
	f.main.background = t
	
	self.frame = f
	
	self:UpdateCandleCount()
end

function SacredBuff:ScaleFrame()
	self:SavePosition()
	_G["SacredBuffFrame"]:SetScale(self.db.profile.scaleSize)
	self:RestorePosition()
end

function SacredBuff:SavePosition()
	local f = _G["SacredBuffFrame"]
	local x, y = f:GetLeft(), f:GetTop()
	local s = f:GetEffectiveScale()
	x, y = ( x * s ), ( y * s )
	self.db.profile.PosX = x
	self.db.profile.PosY = y
end

function SacredBuff:RestorePosition()
	local x = SacredBuff.db.profile.PosX or 0
	local y = SacredBuff.db.profile.PosY or 0
	local f = _G["SacredBuffFrame"]
	local s = f:GetEffectiveScale()
	x, y = ( x / s ), ( y / s )
	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
end

local function checkCooldown(self, spell, mode)
	local SetDesaturation = SetDesaturation
	local start, duration = GetSpellCooldown(spell)
	if start and duration then
		if start > 1 and duration > 1 and not self.desaturated then
			SetDesaturation(self.icon, true)
			self.desaturated = true
		elseif start == 0 and duration == 0 and self.desaturated then
			SetDesaturation(self.icon, false)
			self.desaturated = false
		end
	end
end

local function checkUsable(self, spell, mode)
	local isUsable, notEnoughMana = IsUsableSpell(spell)
	if not isUsable and notEnoughMana then
		self.icon:SetVertexColor(0.10, 0.30, 1.00)
		self.outOfMana = true
	elseif self.outOfMana and not notEnoughMana then
		if self.outOfRange then
			self.icon:SetVertexColor(0.80, 0.10, 0.10)
		else
			self.icon:SetVertexColor(1.00, 1.00, 1.00)
		end
		self.outOfMana = nil
	end
end

function SacredBuff:UpdateButtonsUsable()
	local mode = self.db.char.groupMode
	local spell
	for k, v in next, self.frame do
		if self.db.char.spells[k] and v.states then
			spell = v.states[mode] or v.states[1]
			checkCooldown(v, spell, mode)
			checkUsable(v, spell, mode)
		end
	end
end

function SacredBuff:MakeButtonFrame(x, y)
	local f = CreateFrame("Button", nil, self.frame.main, "SecureActionButtonTemplate")
	f:SetWidth(35)
	f:SetHeight(35)
	f:EnableMouse(true)
	f:SetFrameStrata("MEDIUM")
	f:SetToplevel(true)
	f:SetNormalTexture(TEX_SERENITY)
	f:SetHighlightTexture(TEX_SERENITY)
	f:SetPushedTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	f:RegisterForClicks("AnyUp")
	
	f:ClearAllPoints()
	f:SetPoint("CENTER", self.frame.main, "CENTER", tonumber(x), tonumber(y))
	
	f:SetScript("OnLeave", self.EmptyTooltip)
	
	local t = f:CreateTexture(nil, "BACKGROUND")
	t:SetDrawLayer("BACKGROUND")
	t:SetWidth(26)
	t:SetHeight(26)
	t:ClearAllPoints()
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	f.icon = t
	
	t = f:CreateTexture(nil, "OVERLAY")
	t:SetWidth(40)
	t:SetHeight(40)
	t:SetTexture(TEX_SERENITY)
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	f.icon.overlay = t
	
	t = f:GetNormalTexture()
	t:SetWidth(40)
	t:SetHeight(40)
	t:SetDrawLayer("BACKGROUND")
	t:ClearAllPoints()
	t:SetPoint("CENTER", f.icon, "CENTER", 0, 0)
	f:SetNormalTexture(t)
	f.normal = t
	
	t = f:GetHighlightTexture()
	t:SetVertexColor( 75/255, 216/255, 241/255 )
	t:SetWidth(40)
	t:SetHeight(40)
	t:ClearAllPoints()
	t:SetBlendMode("BLEND")
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	f:SetHighlightTexture(t)
	f.highlight = t
	
	t = f:GetPushedTexture()
	t:SetWidth(35)
	t:SetHeight(35)
	t:ClearAllPoints()
	t:SetPoint("CENTER", f, "CENTER", 0, 0)
	t:SetDrawLayer("HIGHLIGHT")
	t:SetBlendMode("ADD")
	f:SetPushedTexture(t)
	f.pushed = t
	
	return f
end

function SacredBuff:NewActionButton(x, y, singleSpell, partySpell)
	local f = self:MakeButtonFrame(x, y)
	
	self.frame:SetAttribute("addchild", f)
	f:SetAttribute("useparent-statebutton", true)
	f:SetAttribute("useparent-unit", true)
	if singleSpell then
		f:SetAttribute("*type*", "spell")
		f:SetAttribute("*spell-singletarget", singleSpell)
		f:SetAttribute("*spell-singleplayer", singleSpell)
		f.states = { singleSpell }
		local spellCheck
		if partySpell then
			spellCheck = GetSpellTexture(partySpell)
		end
		if not spellCheck or not partySpell then
			spellCheck = singleSpell
		elseif partySpell and spellCheck then
			spellCheck = partySpell
		end
		f:SetAttribute("*spell-grouptarget", spellCheck)
		f:SetAttribute("*spell-groupplayer", spellCheck)
		if spellCheck ~= singleSpell then
			table.insert(f.states, spellCheck)
		end
		spellCheck = f.states[self.db.char.groupMode] or f.states[1]
		local texture = GetSpellTexture(spellCheck)
		if not texture then
			error(("SacredBuff: Attempt to get texture for %s."):format(spellCheck))
		end
		f.icon:SetTexture(texture)
		f:SetScript("OnEnter", self.OnButtonEnter)
	end
	
	if not self:IsActive() then
		f:Hide()
	end
	
	if not self.metroStarted then
		local registered, _, running = M:Status("SacredBuff_ButtonsUsable")
		if registered and not running then
			M:Start("SacredBuff_ButtonsUsable")
			self.metroStarted = true
		end
	end
	
	return f
end

function SacredBuff:OnButtonEnter()
	local mode, spellTable = SacredBuff.db.profile.tooltip
	if mode == 0 then
		return
	end
	GameTooltip:SetOwner(self)
	if SacredBuff.frame.TALENT and self == SacredBuff.frame.TALENT then
		spellTable = SacredBuff.db.char.spells.TALENT
	elseif SacredBuff.frame.RACIAL and self == SacredBuff.frame.RACIAL then
		spellTable = SacredBuff.db.char.spells.RACIAL
	else
		if self.states[ SacredBuff.db.char.groupMode ] then
			spellTable = SacredBuff.db.char.spells[ self.states[ SacredBuff.db.char.groupMode ] ]
		else
			spellTable = SacredBuff.db.char.spells[ self.states[1] ]
		end
	end
	if spellTable then
		if mode == 1 then
			GameTooltip:AddLine(spellTable[3])
		elseif mode == 2 then
			GameTooltip:SetSpell(spellTable[1], BOOKTYPE_SPELL)
		end
		GameTooltip:Show()
	end
end

function SacredBuff:EmptyTooltip()
	local f = GameTooltip
	f:ClearLines()
	local text
	for i = 1, 30 do
		text = _G["SacredBuffTooltipTextRight"..i]
		if text then
			text:SetText(nil)
		end
		text = nil
	end
	f:SetOwner(UIParent, "ANCHOR_NONE")
	f:Hide()
end

function SacredBuff:UpdateButtons()
	local CHAR_DB = self.db.char
	local FRAME_TBL = self.frame
	if CHAR_DB.playerLevel > 39 then
		if not FRAME_TBL.mount then
			local f = self:NewActionButton(-32, -32)
			f:SetAttribute("type", "item")
			FRAME_TBL.mount = f
		end
		self:UpdateMountLocs()
		local tbl
		if GetRealZoneText() == L["Ahn'Qiraj"] then
			tbl = "prefAQMount"
		else
			tbl = "prefMount"
		end
		if CHAR_DB[tbl] then
			local bag, slot = CHAR_DB[tbl][2], CHAR_DB[tbl][3]
			FRAME_TBL.mount.icon:SetTexture( GetContainerItemInfo(bag, slot) )
			FRAME_TBL.mount:SetAttribute("bag", bag)
			FRAME_TBL.mount:SetAttribute("slot", slot)
			FRAME_TBL.mount:Show()
		else
			FRAME_TBL.mount:Hide()
		end
	end
	
	CHAR_DB = CHAR_DB.spells
	if CHAR_DB[ BS["Inner Fire"] ] then
		if not FRAME_TBL[ BS["Inner Fire"] ] then
			FRAME_TBL[ BS["Inner Fire"] ] = self:NewActionButton(32, -32, BS["Inner Fire"])
		end
	end
	
	if CHAR_DB[ BS["Resurrection"] ] then
		if not FRAME_TBL[ BS["Resurrection"] ] then
			FRAME_TBL[ BS["Resurrection"] ] = self:NewActionButton(0, -45, BS["Resurrection"])
		end
	end
	
	if CHAR_DB["RACIAL"] then
		if not FRAME_TBL["RACIAL"] then
			FRAME_TBL["RACIAL"] = self:NewActionButton(45, 0, self.db.char.spells["RACIAL"][4])
		end
	end
	
	if CHAR_DB["TALENT"] then
		if not FRAME_TBL["TALENT"] then
			FRAME_TBL["TALENT"] = self:NewActionButton(-45, 0, self.db.char.spells["TALENT"][4])
		end
	end
	
	if CHAR_DB[ BS["Divine Spirit"] ] then
		if not FRAME_TBL[ BS["Divine Spirit"] ] then
			FRAME_TBL[ BS["Divine Spirit"] ] = self:NewActionButton(-32, 32, BS["Divine Spirit"], BS["Prayer of Spirit"])
		end
	end
	
	if CHAR_DB[ BS["Power Word: Fortitude"] ] then
		if not FRAME_TBL[ BS["Power Word: Fortitude"] ] then
			FRAME_TBL[ BS["Power Word: Fortitude"] ] = self:NewActionButton(0, 45, BS["Power Word: Fortitude"], BS["Prayer of Fortitude"])
		end
	end
	
	if CHAR_DB[ BS["Shadow Protection"] ] then
		if not FRAME_TBL[ BS["Shadow Protection"] ] then
			FRAME_TBL[ BS["Shadow Protection"] ] = self:NewActionButton(32, 32, BS["Shadow Protection"], BS["Prayer of Shadow Protection"])
		end
	end
	
end
